#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "gui.h"
#include "pref.h"
#include "plot.h"
#include "main.h"

GtkTreeView *antdescr_view = NULL;

typedef struct {
  int width;
  char type;
  int colnr;
  gfloat align;
  char *title;
} antdescr_column;

typedef struct {
  char *widget;
  char *color;
} area_descr_record;

area_descr_record area_descr[] = {
  {"xy_darea", "#e8e8ff"},
  {"xz_darea", "#e8ffe8"},
  {"yz_darea", "#ffe8e8"},
  {NULL}
};

antdescr_column antdescr_col[] = {
  {60,	't',    ANT_EXC_MAG,	1.0,	"Magn."},
  {60,	't',    ANT_EXC_PHA,	1.0,	"Phase"},
  {60,	't',    ANT_POS_X,	1.0,	"X"},
  {60,	't',    ANT_POS_Y,	1.0,	"Y"},
  {60,	't',    ANT_POS_Z,	1.0,	"Z"},
  {12,	'r',    ANT_LAST,	0.5,	"!"},
  {-1}
};


//-----------------------------------------------------------------------------
//	Auxiliary routines

void
set_selected_as_last(void)
{
  GtkTreeModel *descriptions;
  GtkTreeSelection *sel;
  GtkTreeIter iter;
  gboolean ok, last;

  descriptions = gtk_tree_view_get_model(antdescr_view);
  sel = gtk_tree_view_get_selection(antdescr_view);
  ok = gtk_tree_model_get_iter_first(descriptions, &iter);
  while (ok) {
    gtk_list_store_set(GTK_LIST_STORE(descriptions), &iter,
       		ANT_LAST, FALSE,
                -1);
    ok = gtk_tree_model_iter_next(descriptions, &iter);
  }
  if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
    gtk_list_store_set(GTK_LIST_STORE(descriptions), &iter,
                ANT_LAST, TRUE,
                -1);
  }
}

void
last_dipole_toggled(gpointer data)
{
  set_selected_as_last();
  redraw_plot();
}

void
initialize_antenna_list(void)
{
  GtkListStore *descriptions;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *col;
  int def, f, tot_width;
             
  descriptions = gtk_list_store_new(ANT_NR_COLS,
    G_TYPE_STRING,		// Excitation (magnitude)
    G_TYPE_STRING,		// Excitation (phase)
    G_TYPE_STRING,		// Antenna position (X)
    G_TYPE_STRING,		// Antenna position (Y)
    G_TYPE_STRING,		// Antenna position (Z)
    G_TYPE_BOOLEAN,		// Last element to analyze
    G_TYPE_POINTER		// Pointer to binary representation
  );

  antdescr_view = GTK_TREE_VIEW(lookup_widget(MainWindow, "description_tview"));
  gtk_tree_view_set_model(antdescr_view, GTK_TREE_MODEL(descriptions));
  g_object_unref(G_OBJECT(descriptions));

  tot_width = 0;
  for (f = 0; antdescr_col[f].width >= 0; f++) {
    switch (antdescr_col[f].type) {
      case 't': // Text renderer
           renderer = gtk_cell_renderer_text_new();
           gtk_object_set(GTK_OBJECT(renderer),
                "ypad", 0,
                "xalign", antdescr_col[f].align,
                NULL);
           col = gtk_tree_view_column_new_with_attributes(_(antdescr_col[f].title),
                renderer, 
                "text", antdescr_col[f].colnr, 
                NULL);  
           break;
            
      case 'r': // Radio button (uses toggle renderer)
           renderer = gtk_cell_renderer_toggle_new();
           g_object_set(G_OBJECT(renderer),
                "ypad", 0,
                "activatable", TRUE,
                "radio", TRUE,
                NULL);
           g_signal_connect(G_OBJECT(renderer), "toggled",
                G_CALLBACK(last_dipole_toggled), GINT_TO_POINTER(f));
           col = gtk_tree_view_column_new_with_attributes(_(antdescr_col[f].title),
                renderer, 
                "active", antdescr_col[f].colnr, 
                NULL);
           break;
    };
      
    gtk_tree_view_append_column(GTK_TREE_VIEW(antdescr_view), col);
    gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(col),
                GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(col),
                antdescr_col[f].width);
    tot_width += antdescr_col[f].width + 1;
  }
  gtk_widget_set_size_request(GTK_WIDGET(antdescr_view), tot_width, -1);
}

void
antenna_list_add(double magn, double phase, double x, double y, double z, int is_last)
{
  GtkListStore *descriptions;
  GtkTreeIter iter;
  GtkWidget *w;
  dipole_def *def = malloc(sizeof(dipole_def));
  char *p[5];
  int i;
  
  def->v = to_rect(magn, to_rad(phase));
  def->x = x; def->y = y; def->z = z;

  descriptions = GTK_LIST_STORE(gtk_tree_view_get_model(antdescr_view));
  gtk_list_store_append(descriptions, &iter);
  gtk_list_store_set(descriptions, &iter,
		ANT_EXC_MAG, p[0] = g_strdup_printf("%.3f", magn),
		ANT_EXC_PHA, p[1] = g_strdup_printf("%.3f", phase),
		ANT_POS_X,   p[2] = g_strdup_printf("%.3f", x),
		ANT_POS_Y,   p[3] = g_strdup_printf("%.3f", y),
		ANT_POS_Z,   p[4] = g_strdup_printf("%.3f", z),
		ANT_LAST,    is_last,
		ANT_RECORD,  def,
		-1
  );
  for (i = 1; i < 5; i++)
    g_free(p[i]);
}

void
antenna_list_clear(void)
{
  GtkTreeModel *model = gtk_tree_view_get_model(antdescr_view);
  GtkTreeIter iter;
  gboolean ok;
  dipole_def *p;
  
  ok = gtk_tree_model_get_iter_first(model, &iter);
  while (ok) {
    gtk_tree_model_get(model, &iter, ANT_RECORD, &p, -1);
    g_free(p);
    ok = gtk_tree_model_iter_next(model, &iter);
  }
  
  gtk_list_store_clear(GTK_LIST_STORE(model));
}

//-----------------------------------------------------------------------------
//	Signal handlers - Menu bar

//	Menu bar - File

void
new1_activate(void)
{
}

void
open1_activate(void)
{
  if (run_filedialog("Load design", &pref.loaded_file, TRUE, 
	"Array files", "*.arr", 
	"All files", "*",
	NULL)) {
    load_design();
  }
}

void
save1_activate(void)
{
  if (pref.loaded_file)
    save_design();
}

void
save_as1_activate(void)
{
  if (run_filedialog("Save design", &pref.loaded_file, FALSE,
	"Array files", "*.arr", 
	"All files", "*",
	NULL)) {
    save_design();
  }
}

void
quit1_activate(void)
{
  gtk_exit(0);
}

//	Menu bar - Edit

void
cut1_activate(void)
{
}

void
copy1_activate(void)
{
}

void
paste1_activate(void)
{
}

void
delete1_activate(void)
{
}

//	Menu bar - Help

void
about1_activate(void)
{
  GtkAboutDialog *dlg = GTK_ABOUT_DIALOG(gtk_about_dialog_new());
  
  gtk_about_dialog_set_program_name(dlg, prog_name);
  gtk_about_dialog_set_version(dlg, prog_version);
  gtk_about_dialog_set_copyright(dlg, prog_copyright);
  gtk_about_dialog_set_license(dlg, prog_license);
  gtk_about_dialog_set_authors(dlg, authors);
  gtk_about_dialog_set_wrap_license(dlg, TRUE);
  
  gtk_dialog_run(GTK_DIALOG(dlg));
  gtk_widget_destroy(GTK_WIDGET(dlg));
  
}

//	Menu bar - View

void
xy_plane1_activate(void)
{
  gtk_notebook_set_page(GTK_NOTEBOOK(lookup_widget(MainWindow, "plot_notebook")), 0);
}

void
zx_plane1_activate(void)
{
  gtk_notebook_set_page(GTK_NOTEBOOK(lookup_widget(MainWindow, "plot_notebook")), 1);
}

void
yz_plane1_activate(void)
{
  gtk_notebook_set_page(GTK_NOTEBOOK(lookup_widget(MainWindow, "plot_notebook")), 2);
}

void
threed_view1_activate(void)
{
  gtk_notebook_set_page(GTK_NOTEBOOK(lookup_widget(MainWindow, "plot_notebook")), 3);
}

void
stereo_view1_activate(void)
{
}

void
linear_scale1_activate(void)
{
  set_plot_scale(PLOT_SCALE_LIN);
}

void
logv_scale1_activate(void)
{
  set_plot_scale(PLOT_SCALE_LOGV);
}

void
logp_scale1_activate(void)
{
  set_plot_scale(PLOT_SCALE_LOGP);
}

void
arrl_scale1_activate(void)
{
  set_plot_scale(PLOT_SCALE_ARRL);
}

void
scale_linear_btn_clicked(void)
{
  set_plot_scale(PLOT_SCALE_LIN);
}

void		scale_logv_btn_clicked(void)
{
  set_plot_scale(PLOT_SCALE_LOGV);
}

void		scale_logp_btn_clicked(void)
{
  set_plot_scale(PLOT_SCALE_LOGP);
}

void		scale_ARRL_btn_clicked(void)
{
  set_plot_scale(PLOT_SCALE_ARRL);
}

void
main_scale_btn_clicked(int which)
{
  set_plot_scale(which);
}


//	Widget events

gboolean
MainWindow_delete_event(void)
{
  gtk_exit(0);
  return FALSE;
}

void
MainWindow_realize(void)
{
  GtkWidget *w;
  GdkColor bgcolor;
  int a;

  for (a = 0; area_descr[a].widget; a++) {
    w = lookup_widget(MainWindow, area_descr[a].widget);
    gdk_color_parse(area_descr[a].color, &bgcolor);
    gtk_widget_modify_bg(w, GTK_STATE_NORMAL, &bgcolor);
  }

  initialize_antenna_list();

  antenna_list_add(1., 0., -0.75, 0.0, 0.0, FALSE);	// Add initial dipole
  antenna_list_add(3., 0., -0.25, 0.0, 0.0, FALSE);	// Add another dipole
  antenna_list_add(3., 0.,  0.25, 0.0, 0.0, FALSE);	// Add another dipole
  antenna_list_add(1., 0.,  0.75, 0.0, 0.0, TRUE);	// Add another dipole

  gtk_widget_realize(lookup_widget(MainWindow, "threed_viewport"));
}

gboolean
MainWindow_window_state_event(GdkEventWindowState *event)
{
}

gboolean
plot_notebook_button_press_event(GdkEventButton *event)
{

  return FALSE;
}

GList *
specs_from_liststore(int all)
{
  GList *list = NULL;
  GtkTreeIter iter;
  GtkListStore *store;
  gboolean next, last;
  dipole_def *def;

  store = GTK_LIST_STORE(gtk_tree_view_get_model(antdescr_view));

  next = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
  while (next) {
    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
		ANT_RECORD, &def,
                ANT_LAST,   &last,
		-1);
    list = g_list_append(list, def);
    if (last && !all)
      next = FALSE;
    else
      next = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
  }
  return list;
}


void
mark_as_last1_activate(gpointer data)
{
  set_selected_as_last();
  redraw_plot();
}

char *
format_double(double d)
{
  return g_strdup_printf("%.3f", d);
}

void
format_dipole_fields(GtkListStore *store, GtkTreeIter *iter, dipole_def *dip)
{
  char *mag, *pha, *x, *y, *z;
  
  gtk_list_store_set(store, iter,
		ANT_EXC_MAG, mag = format_double(cabs(dip->v)),
		ANT_EXC_PHA, pha = format_double(to_deg(carg(dip->v))),
		ANT_POS_X,   x   = format_double(dip->x),
		ANT_POS_Y,   y   = format_double(dip->y),
		ANT_POS_Z,   z   = format_double(dip->z),
		ANT_RECORD,  dip,
		-1);
  g_free(mag); g_free(pha); g_free(x); g_free(y); g_free(z);
}

gboolean
run_dipole_editor(dipole_def *dip)
{
  GtkWidget *dlg = create_edit_dipole_dialog(), *w;
  GtkEntry *emag, *epha, *ex, *ey, *ez;
  char *mag, *pha, *x, *y, *z;
  char *p;
  int i;
  
  w = lookup_widget(dlg, "edit_dip_polar_rbtn");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
  
  emag = GTK_ENTRY(lookup_widget(dlg, "edit_dip_mag_entry"));
  epha = GTK_ENTRY(lookup_widget(dlg, "edit_dip_phase_entry"));
  ex   = GTK_ENTRY(lookup_widget(dlg, "edit_dip_x_entry"));
  ey   = GTK_ENTRY(lookup_widget(dlg, "edit_dip_y_entry"));
  ez   = GTK_ENTRY(lookup_widget(dlg, "edit_dip_z_entry"));

  gtk_entry_set_text(emag, mag = format_double(creal(dip->v)));
  gtk_entry_set_text(epha, pha = format_double(cimag(dip->v)));
  gtk_entry_set_text(ex,   x   = format_double(dip->x));
  gtk_entry_set_text(ey,   y   = format_double(dip->y));
  gtk_entry_set_text(ez,   z   = format_double(dip->z));

  g_free(mag); g_free(pha); g_free(x); g_free(y); g_free(z);

  if (gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_OK) {
    dip->v = to_rect(       strtod(gtk_entry_get_text(emag), NULL), 
                     to_rad(strtod(gtk_entry_get_text(epha), NULL)));
    dip->x = strtod(gtk_entry_get_text(ex), NULL);
    dip->y = strtod(gtk_entry_get_text(ey), NULL);
    dip->z = strtod(gtk_entry_get_text(ez), NULL);
    
    gtk_widget_destroy(dlg);
    return TRUE;
  }
  gtk_widget_destroy(dlg);
  return FALSE;
}


void
edit_dipole1_activate(void)
{
  GtkWidget *w;
  GtkTreeModel *descriptions;
  GtkTreeSelection *sel;
  GtkTreeIter iter;
  dipole_def *dip;

  sel = gtk_tree_view_get_selection(antdescr_view);
  gtk_tree_selection_get_selected(sel, &descriptions, &iter);
  gtk_tree_model_get(descriptions, &iter,
                ANT_RECORD, &dip,
                -1);

  if (run_dipole_editor(dip))
    format_dipole_fields(GTK_LIST_STORE(descriptions), &iter, dip);
  redraw_plot();
}

void
delete_dipole1_activate(void)
{
  GtkTreeModel *model;
  GtkTreeSelection *sel;
  GtkTreeIter iter;
  gboolean ok, last;

  sel = gtk_tree_view_get_selection(antdescr_view);
  if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
  }
  redraw_plot();
}

void
add_dipole1_activate(void)
{
  GtkWidget *w;
  GtkTreeModel *descriptions;
  GtkTreeIter iter;
  dipole_def *dip = (dipole_def *) malloc(sizeof(dipole_def));

  dip->v = 1.0 + I * 0.0;
  dip->x = dip->y = dip->z = 0.0;
  
  if (run_dipole_editor(dip)) {
    w = lookup_widget(MainWindow, "description_tview");
    descriptions = gtk_tree_view_get_model(GTK_TREE_VIEW(w));
    gtk_list_store_append(GTK_LIST_STORE(descriptions), &iter);
    format_dipole_fields(GTK_LIST_STORE(descriptions), &iter, dip);
  }
  redraw_plot();
}

void
move_dipole_up_activate(void)
{
  GtkTreeView *tview;
  GtkTreeModel *descriptions;
  GtkTreeSelection *sel;
  GtkTreeIter iter, active_iter, *prev_iter;
  GtkTreePath *active_path, *path;
  int ok;
  
  tview = GTK_TREE_VIEW(lookup_widget(MainWindow, "description_tview"));
  sel = gtk_tree_view_get_selection(tview);
  
  if (gtk_tree_selection_get_selected(sel, &descriptions, &active_iter)) {
    active_path = gtk_tree_model_get_path(descriptions, &active_iter);

    prev_iter = NULL;
    ok = gtk_tree_model_get_iter_first(descriptions, &iter);
    while (ok) {
      path = gtk_tree_model_get_path(descriptions, &iter);
      if (gtk_tree_path_compare(active_path, path) == 0)
        break;
        
      if (prev_iter) 
        gtk_tree_iter_free(prev_iter);
        
      prev_iter = gtk_tree_iter_copy(&iter);
      ok = gtk_tree_model_iter_next(descriptions, &iter);
    }
    
    if (prev_iter == NULL)
      return;

    gtk_list_store_swap(GTK_LIST_STORE(descriptions), &active_iter, prev_iter);
    gtk_tree_iter_free(prev_iter);
  } 
  redraw_plot();
}

void
move_dipole_down_activate(void)
{
  GtkTreeView *tview;
  GtkTreeSelection *sel;
  GtkTreeIter *next_iter, active_iter;
  GtkTreeModel *descriptions;

  tview = GTK_TREE_VIEW(lookup_widget(MainWindow, "description_tview"));
  sel = gtk_tree_view_get_selection(tview);
  if (!gtk_tree_selection_get_selected(sel, &descriptions, &active_iter))
    return;

  next_iter = gtk_tree_iter_copy(&active_iter);
  if (gtk_tree_model_iter_next(descriptions, next_iter)) {
    gtk_list_store_swap(GTK_LIST_STORE(descriptions), &active_iter, next_iter);
  }
  gtk_tree_iter_free(next_iter);
  redraw_plot();
}

void
edit_dip_polar_rbtn_toggled(GtkToggleButton *button)
{
  GtkWidget *wmag, *wphase, *wre, *wim;
  gboolean polar;

  polar = gtk_toggle_button_get_active(button);

  wmag =   lookup_widget(GTK_WIDGET(button), "edit_dip_mag_entry");
  wphase = lookup_widget(GTK_WIDGET(button), "edit_dip_phase_entry");
  wre = lookup_widget(GTK_WIDGET(button), "edit_dip_re_entry");
  wim = lookup_widget(GTK_WIDGET(button), "edit_dip_im_entry");
    
  gtk_widget_set_sensitive(wmag,   polar);
  gtk_widget_set_sensitive(wphase, polar);
  gtk_widget_set_sensitive(wre, !polar);
  gtk_widget_set_sensitive(wim, !polar);

  if (polar) {
    gtk_widget_grab_focus(wmag);
    gtk_editable_select_region(GTK_EDITABLE(wmag), 0, -1);
  } else {
    gtk_widget_grab_focus(wre);
    gtk_editable_select_region(GTK_EDITABLE(wre), 0, -1);
  }
}


void
set_fullscreen_mode(gboolean full)
{
  if (full) {
    gtk_window_fullscreen(GTK_WINDOW(MainWindow));
  } else {
    gtk_window_unfullscreen(GTK_WINDOW(MainWindow));
  }
}

//  Element list operations

void
el_list_remove_btn_clicked(GtkButton *btn)
{
  delete_dipole1_activate();
}

void
el_list_add_btn_clicked(GtkButton *btn)
{
  add_dipole1_activate();
}

void
el_list_edit_btn_clicked(GtkButton *btn)
{
  edit_dipole1_activate();
}

void
el_list_move_up_btn_clicked(GtkButton *btn)
{
  move_dipole_up_activate();
}

void
el_list_move_down_btn_clicked(GtkButton *btn)
{
  move_dipole_down_activate();
}

void
el_list_mark_last_btn_clicked(GtkButton *btn)
{
  mark_as_last1_activate(NULL);
}

void
fullscreen1_activate(GtkMenuItem *item)
{
  set_fullscreen_mode(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item)));
}
