#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <libxml/tree.h>
#include <support.h>

#include "main.h"
#include "pref.h"
#include "gui.h"
#include "plot.h"

preferences pref = {
  PLOT_SCALE_ARRL,
  VIEW_3D_LINE,
  NULL			// No loaded file yet
};

typedef struct {
  const char *caption;
  int par;
  void (*func)(int);
} option_entry;

option_entry options[] = {
  {"<b>View</b>", 0, NULL},
  {"X-Y plane", VIEW_XY, set_view_dir},
  {"Y-Z plane", VIEW_YZ, set_view_dir},
  {"Z-X plane", VIEW_ZX, set_view_dir},
  
  {"<b>Draw</b>", 0, NULL},
  {"Graph",     VIEW_GRAPH,     set_view},
  {"3D Lines",  VIEW_3D_LINE,   set_view},
  {"3D Fill",   VIEW_3D_FILL,   set_view},
  {"Anaglyph",  VIEW_ANAGLYPH,  set_view},
  
  {"<b>Scale</b>", 0, NULL},
  {"Linear",    PLOT_SCALE_LIN,	 set_plot_scale},
  {"log(V)",    PLOT_SCALE_LOGV, set_plot_scale},
  {"log(P)",    PLOT_SCALE_LOGP, set_plot_scale},
  {"ARRL",      PLOT_SCALE_ARRL, set_plot_scale},
  {NULL}
};

void
save_design(void)
{
  GList *list, *lp;
  FILE *xmlf;
  dipole_def *dip;

  xmlf = fopen(pref.loaded_file, "w");
  if (!xmlf) {
    printf("Cannot open %s to write.\n", pref.loaded_file);
    return;
  }

  list = specs_from_liststore(TRUE);	// Get all dipoles
  fprintf(xmlf, "<?xml version=\"1.0\"?>\n");
  fprintf(xmlf, "<" XML_TYPE ">\n");
  fprintf(xmlf, " <version value=\"%s\" />\n", prog_version);

  for (lp = g_list_first(list); lp; lp = lp->next) {
    dip = (dipole_def *) lp->data;
    fprintf(xmlf, " <dipole x=\"%f\" y=\"%f\" z=\"%f\" mag=\"%f\" phase=\"%f\" />\n",
                  dip->x, dip->y, dip->z, cabs(dip->v), carg(dip->v)); 
  }

  fprintf(xmlf, "</dipole_array>\n");
  fclose(xmlf);
}

void
parse_element(dipole_def *dip, xmlDocPtr doc, xmlNodePtr cur)
{
  char *p;
  real mag, pha;
  
  dip->x = atof(p = xmlGetProp(cur, "x")); xmlFree(p);
  dip->y = atof(p = xmlGetProp(cur, "y")); xmlFree(p);
  dip->z = atof(p = xmlGetProp(cur, "z")); xmlFree(p);
  mag = atof(p = xmlGetProp(cur, "mag")); xmlFree(p);
  pha = atof(p = xmlGetProp(cur, "phase")); xmlFree(p);
  dip->v = to_rect(mag, to_rad(pha));
}

gboolean
load_design(void)
{
  xmlNodePtr cur;
  xmlDocPtr doc;
  dipole_def dip;

  doc = xmlParseFile(pref.loaded_file);
  if (doc == NULL ) {
    ok_dialog("Document not parsed successfully.");
    return FALSE;   
  }
      
  cur = xmlDocGetRootElement(doc);
        
  if (cur == NULL) {
    ok_dialog("Empty document");
    xmlFreeDoc(doc);
    return FALSE;
  }
        
  if (xmlStrcmp(cur->name, (const xmlChar *) XML_TYPE)) {
    ok_dialog("Document of the wrong type.");
    xmlFreeDoc(doc);
    return FALSE;   
  }

  antenna_list_clear();
  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *) XML_DIPOLE))) {
      parse_element(&dip, doc, cur);
      antenna_list_add(cabs(dip.v), to_deg(carg(dip.v)), 
                       dip.x, dip.y, dip.z, 
                       FALSE);
    }
    cur = cur->next;
  }  
  xmlFreeDoc(doc);
   
  return TRUE;
}


void
option_clicked(GtkTreeSelection *sel, gpointer data)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  void (*func)(int);
  int par;
  
  if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
    gtk_tree_model_get(model, &iter,
		1, &par,
		2, &func,
		-1);
    if (func) {
      func(par);
    }
  }
}

void
set_view(int view)
{
  pref.plot_view = view;
  switch (view) {
    case VIEW_GRAPH:
      xy_plane1_activate();
      break;
    case VIEW_3D_LINE:
    case VIEW_3D_FILL:
      threed_view1_activate();
      redraw_plot();
      break;
    case VIEW_ANAGLYPH:
      break;
  }
}

void
set_view_dir(int dir)
{
  switch (dir) {
    case VIEW_XY: xy1_activate(); break;
    case VIEW_YZ: yz1_activate(); break;
    case VIEW_ZX: zx1_activate(); break;
  }
}

